﻿using FarPoint.CalcEngine;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.CellType;
using FarPoint.Win.Spread.Model;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsFormsApplication13
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.fpSpread1.AllowUserFormulas = true;
            var sheet = this.fpSpread1.ActiveSheet;
            sheet.AddCustomFunction(new IndirectFunctionInfo(sheet.Models.Data as DefaultSheetDataModel));
            sheet.Cells[0, 0].Value = 987;
            sheet.Cells[1, 1].Value = 123;
            sheet.Cells[2, 2].Value = "A1";
            sheet.Cells[3, 3].Formula = "INDIRECT(\"B2\")";
            sheet.Cells[4, 4].Formula = "INDIRECT(C3)";

        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.fpSpread1.SaveExcel("test.xls");
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.fpSpread1.OpenExcel("test.xls");
            this.fpSpread1.ActiveSheet.AddCustomFunction(new IndirectFunctionInfo(this.fpSpread1.ActiveSheet.Models.Data as DefaultSheetDataModel));
            
        }
    }

    [Serializable()]
    class IndirectFunctionInfo : FunctionInfo
    {
        DefaultSheetDataModel Model;
        public IndirectFunctionInfo(DefaultSheetDataModel model)
        {
            Model = model;
        }
        public override string Name
        {
            get { return "INDIRECT"; }
        }
        public override int MinArgs
        {
            get { return 1; }
        }
        public override int MaxArgs
        {
            get { return 1; }
        }
        public override bool AcceptsReference(int i)
        {
            return i == 0;
        }
        public override bool IsVolatile()
        {
            return true;
        }
        public override object Evaluate(object[] args)
        {
            CalcReference range = args[0] as CalcReference;
            object value = range != null ? range.GetValue(range.Row, range.Column) : args[0];
            if (value == null) return null;
            CellExpression expression = Model.ParseFormula(0, 0, value.ToString()) as CellExpression;
            if (expression == null) return null;
            return new CellCalcReference(Model, expression);
        }

        class CellCalcReference : CalcReference
        {
            DefaultSheetDataModel dataModel;
            CellExpression expression;
            public CellCalcReference(DefaultSheetDataModel model, CellExpression ex)
            {
                dataModel = model;
                expression = ex;
            }
            public override int Column
            {
                get { return expression.Column; }
            }
            public override int ColumnCount
            {
                get { return expression.ColumnCount; }
            }
            public override CalcReference GetSource()
            {
                return null;
            }
            public override object GetValue(int row, int column)
            {
                return dataModel.GetValue(row, column);
            }
            public override bool IsSubtotal(int row, int column)
            {
                return false;
            }
            public override int Row
            {
                get { return expression.Row; }
            }
            public override int RowCount
            {
                get { return expression.RowCount; }
            }
        }
    }


}
